-- ***********************************************************************
-- *			'mPlayerInfo' by Rawi				 *
-- *			   Version - 1.1.1				 *
-- * 		Inspired by PlayerInFu by Kayde of Alleria		 *
-- * Armor and Damage Reduction formulas from LibStatLogic by Whitetooth *
-- ***********************************************************************

local _G = getfenv(0)
local string = _G.string
local math = _G.math

local ADDON_NAME	= ...
local LibStub		= _G.LibStub
local mPlayerInfo	= LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "AceEvent-3.0")
local LQT		= LibStub("LibQTip-1.0")
local LDB		= LibStub("LibDataBroker-1.1")
local LDBIcon		= LibStub("LibDBIcon-1.0")
local L			= LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME)
local AB		= LibStub("LibAbacus-3.0")
local CR		= LibStub("LibCrayon-3.0")
local DataObj
local tooltip
--
-- Constants
local defaults = {
	global = {
		datafeed = {
			minimap_icon	= {
				hide	= true,
			},
			show_PlayerN	= true,
			show_PlayerL	= true,
			use_PlayerCC	= false,
			show_PlayerXPP	= true,
			show_PlayerXPN	= false,
			show_PlayerXPR	= true,
			show_PlayerXPRe	= true,			
		},
		tooltip = {
			hide_Hint	= false,
			show_Exp	= true,
			show_Attributes	= true,
			show_Defense	= true,
			show_Melee	= true,
			show_Ranged	= true,
			show_Spell	= true,
			show_Talents	= true,
			scale		= 1,
			timer		= 0.25,
		},
	}
}
local max_level = 85
local prof_max_level = 525
--
-- Variables.
local db
local playerClass, englishClass = UnitClass("player")
local startXP, startTime, startLevel
--
-- Functions
local function GetPlayerData()
	local c_color = RAID_CLASS_COLORS[englishClass]
	local hex = string.format("|cff%02x%02x%02x", c_color.r * 255, c_color.g * 255, c_color.b * 255)
	local cXP, mXP, rXP, cLEV = UnitXP("player"), UnitXPMax("player"), GetXPExhaustion(), UnitLevel("player")
	local pName, _pLVL ,numbXP, percXP, restXP, remXP = "", "", "", "", "", ""
	local pL = " "
	local CSXP = cXP - startXP
	local CSLev = cLEV + (cXP/mXP) - startLevel
	local CSTime = AB:FormatDurationFull(GetTime() - startTime)

	if db.datafeed.show_PlayerN then
		if db.datafeed.use_PlayerCC then	
			pName = string.format("%s%s|r", hex, UnitName("player"))
		else
			pName = CR:Gold(UnitName("player"))
		end
	end
	
	if db.datafeed.show_PlayerL then
		if db.datafeed.use_PlayerCC then
			_pLVL = string.format("%s%s|r", hex, cLEV)
			pL = string.format("%s%s|r", hex, " Lvl ")
		else
			_pLVL = CR:Gold(cLEV)
			pL = CR:Gold(" Lvl ")
		end
		if db.datafeed.show_PlayerN then
			if db.datafeed.use_PlayerCC then
				pL = string.format("%s%s|r", hex, " - Lvl ")
			else
				pL = CR:Gold(" - Lvl ")
			end
		end			
	end
			
	if not (cLEV == max_level) then
		if db.datafeed.show_PlayerXPN then		
			numbXP = CR:White(format(" [%.f/%.f]", cXP, mXP))			
		end
		if db.datafeed.show_PlayerXPP then		
			percXP = CR:Cyan(format("(%.1f%%)", (cXP / mXP * 100)))			
		end
		if(rXP ~= nil) and (db.datafeed.show_PlayerXPR) then
			restXP = CR:Green(" +"..rXP)			
		end
		if db.datafeed.show_PlayerXPRe then		
			remXP = CR:White(format("%s%c%d%s", " ", "94", mXP - cXP, " XP"))			
		end
	end
	
	local session_LEV = string.format("%.2f", CSLev)
	local session_XP = string.format("%s", CSXP)
	local session_T = string.format("%s", CSTime)	
	local player_C = string.format("%s%s|r", hex, playerClass)
	local color_text = string.format("%s%s%s%s%s%s%s%s|r", pName, pL, _pLVL, " ", percXP, numbXP, restXP, remXP)			
	return session_LEV, session_XP, session_T, player_C, color_text	
end

-- Spell Crit Percentage from all schools  /from PlayerInFu by Kayde of Alleria/
local function GetRealSpellCrit()
	local holySchool = 2
	local minCrit = GetSpellCritChance(holySchool)
	local spellCrit
	mPlayerInfo.spellCrit = {}
	mPlayerInfo.spellCrit[holySchool] = minCrit
	for i=(holySchool+1), 7 do
		spellCrit = GetSpellCritChance(i)
		minCrit = min(minCrit, spellCrit)
		mPlayerInfo.spellCrit[i] = spellCrit
	end
	minCrit = format("%.2f%%", minCrit)
	return minCrit
end

-- Armor Damage Reduction /Formula from LibStatLogic by Whitetooth/
local function GetReductionFromArmor()
	local armor = select(2, UnitArmor("player"))
	local levelMod = UnitLevel("player")
	if ( levelMod > 80 ) then
		levelMod = levelMod + (4.5 * (levelMod - 59)) + (20 * (levelMod - 80));
	elseif ( levelMod > 59 ) then
		levelMod = levelMod + (4.5 * (levelMod - 59))
	end
	local armorReduction = (armor / (85 * levelMod + 400)) / (1 + (armor / (85 * levelMod + 400)))	
	if armorReduction > 0.75 then armorReduction = 0.75 end
	if armorReduction < 0 then armorReduction = 0 end
	return armorReduction * 100
end

-- Resilience Damage Reduction /Formula from LibStatLogic by Whitetooth/
local function GetReductionFromResilience()  
	local H = {
		[80] = 3.2789989471436,
		[81] = 4.3056015014648,
		[82] = 5.6539749145508,
		[83] = 7.4275451660156,
		[84] = 9.7527236938477,
		[85] = 12.8057159423828,
		}
	local rating = GetCombatRating(16)
	local ratingBase = 10.78125
	local level = UnitLevel("player")
	if level >= 80 and level <= 85 then
		return rating/ratingBase/H[level]
	elseif level >= 70 then
		return rating/ratingBase/((82/52)*(131/63)^((level-70)/10))
	elseif level >= 60 then
		return rating/ratingBase/(82/(262-3*level))
	elseif level >= 10 then
		return rating/ratingBase/((level-8)/52)
	else
		return rating/ratingBase/(2/52)
	end
end

local function GetPlayerTalentInfo(t_ID)
	local _, tName, _, _, tPoints, _, _, _ = GetTalentTabInfo(t_ID)
--	local pSpec = GetPrimaryTalentTree()
--	if t_ID == pSpec then tName = CR:Gold(tName) end
	if t_ID == GetPrimaryTalentTree() then tName = CR:Gold(tName) end	
	return tName, tPoints
end

local function GetPlayerProfessionInfo(prof_ID)
	local r, g, b, hex
	local p_name, _, p_skillLevel, p_maxSkillLevel, _, _, _ = GetProfessionInfo(prof_ID)
	r,g,b = CR.GetThresholdColor(p_skillLevel / p_maxSkillLevel, p_skillLevel / p_maxSkillLevel)
	hex = string.format("|cff%02x%02x%02x", r * 255, g * 255, b * 255)
	p_skillLevel = string.format("%s%s|r", hex, p_skillLevel)
	r,g,b = CR.GetThresholdColor(p_maxSkillLevel / prof_max_level, p_maxSkillLevel / prof_max_level)
	hex = string.format("|cff%02x%02x%02x", r * 255, g * 255, b * 255)			
	p_maxSkillLevel = CR:White(" / ")..string.format("%s%s|r", hex, p_maxSkillLevel)
	return p_name, p_skillLevel, p_maxSkillLevel
end
--
-- Tooltip
local DrawTooltip
local updater

-- Header Font
local mHeaderFont = CreateFont("mHeaderFont")
mHeaderFont:CopyFontObject(GameTooltipHeaderText)
mHeaderFont:SetFont(mHeaderFont:GetFont(), 13)
	
local function LDB_OnClick(display, button)
	if button == "RightButton" then
		local options_frame = InterfaceOptionsFrame
		if options_frame:IsVisible() then
			options_frame:Hide()
		else
			InterfaceOptionsFrame_OpenToCategory(mPlayerInfo.options_frame)
		end
	elseif button == "LeftButton" then
		if IsShiftKeyDown() then
			ToggleTalentFrame()
		else
			ToggleCharacter("PaperDollFrame")
		end
	end
end

local function LDB_OnEnter(display, motion)
	DrawTooltip(display)
end

local function LDB_OnLeave()
	updater.elapsed = 0
end

local LDB_anchor
local last_update = 0

updater = CreateFrame("Frame", nil, UIParent)

-- Tooltip hiding
updater:SetScript("OnUpdate", function(self, elapsed)
	  last_update = last_update + elapsed
	  if last_update < 0.1 then
		  return
	  end
	  if tooltip then
		  if tooltip:IsMouseOver() or (LDB_anchor and LDB_anchor:IsMouseOver()) then
			  self.elapsed = 0
		  else
			  self.elapsed = self.elapsed + last_update
			  if self.elapsed >= db.tooltip.timer then
				tooltip = LQT:Release(tooltip)
				LDB_anchor = nil
--				collectgarbage()
			  end
		  end
	  end
	  last_update = 0
end)
--
-- DataObj and Tooltip
function DrawTooltip(anchor)
	LDB_anchor = anchor

	if not tooltip then
		tooltip = LQT:Acquire(ADDON_NAME.."Tooltip", 5, "LEFT", "RIGHT", "CENTER", "LEFT", "RIGHT")
		tooltip:EnableMouse(true)
			if _G.TipTac and _G.TipTac.AddModifiedTip then
			TipTac:AddModifiedTip(tooltip, true)
		end
	end

	local session_level, session_experience, session_time, player_class, player_text = GetPlayerData()
	local pLVL = UnitLevel("player")
	local line
	
	tooltip:Clear()
	tooltip:SmartAnchorTo(anchor)
	tooltip:SetScale(db.tooltip.scale)

	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, player_text, "CENTER", 5)

	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, player_class, mHeaderFont, "CENTER", 5)

	tooltip:AddSeparator()

-- EXPERIENCE
	if db.tooltip.show_Exp then
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Played this session:"], "LEFT", 3)
		tooltip:SetCell(line, 4, session_time, "RIGHT", 2)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Exp. gained this session:"], "LEFT", 3)
		tooltip:SetCell(line, 4, session_experience, "RIGHT", 2)		
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Lev. gained this session:"], "LEFT", 3)
		tooltip:SetCell(line, 4, session_level, "RIGHT", 2)	

		tooltip:AddSeparator()
	end

-- ATTRIBUTES
	if db.tooltip.show_Attributes then
		--STR, AGI, STA, INT, SPI
		local bStr, tStr, pStr, nStr = UnitStat("player",1)
		local bAgi, tAgi, pAgi, nAgi = UnitStat("player",2)
		local bSta, tSta, pSta, nSta = UnitStat("player",3)
		local bInt, tInt, pInt, nInt = UnitStat("player",4)
		local bSpi, tSpi, pSpi, nSpi = UnitStat("player",5)
		
		if pStr == 0 and nStr == 0 then bStr = tStr
		  elseif nStr < 0 then bStr = CR:Red(tStr)
		  else bStr = CR:Green(tStr)
		end
		if pAgi == 0 and nAgi == 0 then bAgi = tAgi
		  elseif nAgi < 0 then bAgi = CR:Red(tAgi)
		  else bAgi = CR:Green(tAgi)
		end
		if pSta == 0 and nSta == 0 then bSta = tSta
		  elseif nSta < 0 then bSta = CR:Red(tSta)
		  else bSta = CR:Green(tSta)
		end
		if pInt == 0 and nInt == 0 then bInt = tInt
		  elseif nInt < 0 then bInt = CR:Red(tInt)
		  else bInt = CR:Green(tInt)
		end
		if pSpi == 0 and nSpi == 0 then bSpi = tSpi
		  elseif nSpi < 0 then bSpi = CR:Red(tSpi)
		  else bSpi = CR:Green(tSpi)
		end

		--ARCANE, FIRE, NATURE, FROST, SHADOW
		local bArc, tArc, pArc, nArc = UnitResistance("player", 6)
		local bFir, tFir, pFir, nFir = UnitResistance("player", 2)
		local bNat, tNat, pNat, nNat = UnitResistance("player", 3)
		local bFst, tFst, pFst, nFst = UnitResistance("player", 4)
		local bShw, tShw, pShw, nShw = UnitResistance("player", 5)

		if pArc == 0 and nArc == 0 then bArc = tArc
		  elseif nArc < 0 then bArc = CR:Red(tArc)
		  else bArc = CR:Green(tArc)
		end
		if pFir == 0 and nFir == 0 then bFir = tFir
		  elseif nFir < 0 then bFir = CR:Red(tFir)
		  else bFir = CR:Green(tFir)
		end
		if pNat == 0 and nNat == 0 then bNat = tNat
		  elseif nNat < 0 then bNat = CR:Red(tNat)
		  else bNat = CR:Green(tNat)
		end
		if pFst == 0 and nFst == 0 then bFst = tFst
		  elseif nFst < 0 then bFst = CR:Red(tFst)
		  else bFst = CR:Green(tFst)
		end
		if pShw == 0 and nShw == 0 then bShw = tShw
		  elseif nShw < 0 then bShw = CR:Red(tShw)
		  else bShw = CR:Green(tShw)
		end

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Attributes"], mHeaderFont, "CENTER", 5)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Strength: "])
		tooltip:SetCell(line, 2, bStr)	
		tooltip:SetCell(line, 4, L["Arcane: "])
		tooltip:SetCell(line, 5, bArc)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Agility: "])
		tooltip:SetCell(line, 2, bAgi)	
		tooltip:SetCell(line, 4, L["Fire: "])
		tooltip:SetCell(line, 5, bFir)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Stamina: "])
		tooltip:SetCell(line, 2, bSta)	
		tooltip:SetCell(line, 4, L["Nature: "])
		tooltip:SetCell(line, 5, bNat)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Intellect: "])
		tooltip:SetCell(line, 2, bInt)	
		tooltip:SetCell(line, 4, L["Frost: "])
		tooltip:SetCell(line, 5, bFst)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Spirit: "])
		tooltip:SetCell(line, 2, bSpi)	
		tooltip:SetCell(line, 4, L["Shadow: "])
		tooltip:SetCell(line, 5, bShw)
		
		tooltip:AddSeparator()	
	end

-- DEFENSE
	if db.tooltip.show_Defense then
		--Health 
		local pMaxHealth = UnitHealthMax("player")
		local pHealth = UnitHealth("player")
		if pHealth < pMaxHealth then pHealth = CR:Red(pHealth).."/"..pMaxHealth end
		--Armor
		local bArmor, eArmor, tArmor, pArmorBuff, nArmorBuff = UnitArmor("player")
		--Armor Damage Reduction
		local aDmgRed = format("%.2f%%", GetReductionFromArmor())
		if pArmorBuff - nArmorBuff < 0 then
			eArmor = CR:Red(format(eArmor))
		elseif bArmor < eArmor then 
			eArmor = CR:Green(format(eArmor))
		end
		--Dodge
		local dodgePer = format("%.2f%%",GetDodgeChance())
		local dodgeComRate = GetCombatRating(3)
		if dodgeComRate > 0 then dodgePer = CR:Green(dodgePer) end
		--Parry
		local parryPer = format("%.2f%%",GetParryChance())
		local parryComRate = GetCombatRating(4)
		if parryComRate > 0 then parryPer = CR:Green(parryPer) end
		--Block
		local blockPer = format("%.2f%%",GetBlockChance())
		local blockComRate = GetCombatRating(5)
		if blockComRate > 0 then blockPer = CR:Green(blockPer) end
		--Resilience Crit/Dmg Reduction
		local resilienceComRate = GetCombatRating(16)
		local resiliencePer = GetReductionFromResilience() 
		resiliencePer = format("%.2f%%", resiliencePer)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Defensive Skills"], mHeaderFont, "CENTER", 5)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Health: "])
		tooltip:SetCell(line, 2, pHealth)	
		tooltip:SetCell(line, 4, L["Dodge: "])
		tooltip:SetCell(line, 5, dodgePer)		

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Block: "])
		tooltip:SetCell(line, 2, blockPer)		
		tooltip:SetCell(line, 4, L["Parry: "])
		tooltip:SetCell(line, 5, parryPer)		
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Armor: "])		
		tooltip:SetCell(line, 2, eArmor)
		tooltip:SetCell(line, 3, format("%s", "-->"))			
		tooltip:SetCell(line, 4, L["Dmg Reduct: "])		
		tooltip:SetCell(line, 5, aDmgRed)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Resilence: "])
		tooltip:SetCell(line, 2, resilienceComRate)
		tooltip:SetCell(line, 3, format("%s", "-->"))		
		tooltip:SetCell(line, 4, L["Dmg Reduct: "])
		tooltip:SetCell(line, 5, resiliencePer)
		
		tooltip:AddSeparator()

	end

-- MELEE
	if db.tooltip.show_Melee then
		--Main/Off Hand Weapon Speed
		local MainSpeed, OffSpeed = UnitAttackSpeed("player")
		local zWeapSpeed = format("%.2f", MainSpeed)
		if OffhandHasWeapon() then
			zWeapSpeed = format("%.2f | %.2f", MainSpeed,OffSpeed)
		end
		--Main/Off Hand Damage Range
		local mLowDmg, mHiDmg, oLowDmg, oHiDmg, pDmgBuff, nDmgBuff, percentMod = UnitDamage("player")
		local tmLowDmg = floor(mLowDmg)
		local tmHiDmg = ceil(mHiDmg)
		local zDmgRng = format("%.f-%.f", tmLowDmg, tmHiDmg)
		if pDmgBuff - nDmgBuff < 0 then
			zDmgRng = CR:Red(format("%.f-%.f", (tmLowDmg + pDmgBuff + nDmgBuff), (tmHiDmg + pDmgBuff + nDmgBuff)))
		elseif pDmgBuff - nDmgBuff > 0 then
			zDmgRng = CR:Green(format("%.f-%.f", (tmLowDmg + pDmgBuff + nDmgBuff), (tmHiDmg + pDmgBuff + nDmgBuff)))
		end
		if OffhandHasWeapon() then
			local toLowDmg = floor(oLowDmg)
			local toHiDmg = ceil(oHiDmg)
			if pDmgBuff - nDmgBuff < 0 then
				zDmgRng = CR:Red(format("%.f-%.f | %.f-%.f", (tmLowDmg + pDmgBuff + nDmgBuff), (tmHiDmg + pDmgBuff + nDmgBuff), (toLowDmg + pDmgBuff + nDmgBuff), (toHiDmg + pDmgBuff + nDmgBuff)))
			elseif pDmgBuff - nDmgBuff > 0 then
				zDmgRng = CR:Green(format("%.f-%.f | %.f-%.f", (tmLowDmg + pDmgBuff + nDmgBuff), (tmHiDmg + pDmgBuff + nDmgBuff), (toLowDmg + pDmgBuff + nDmgBuff), (toHiDmg + pDmgBuff + nDmgBuff)))
			else
				zDmgRng = format("%.f-%.f | %.f-%.f", tmLowDmg, tmHiDmg, toLowDmg, toHiDmg)
			end
		end
		--Main/Off Hand Damage Per Second
		local mDPS = ((tmLowDmg + tmHiDmg) / 2) / MainSpeed
		local zDPS = format("%.1f", mDPS)
		if OffhandHasWeapon() then
			local toLowDmg = floor(oLowDmg)
			local toHiDmg = ceil(oHiDmg)
			local oDPS = ((toLowDmg + toHiDmg) / 2) / OffSpeed
			zDPS = format("%.1f | %.1f", mDPS, oDPS)
		end
		--Melee Crit Chance Percentage
		local zCrit = format("%.2f%%",GetCritChance())
		--Melee Hit Percent Chance versus Same Level NPC/PC
		local zHitRate = "+"..format("%.2f%%", GetCombatRatingBonus(6))
		--Melee Attack Power
		local bAPwr, pAPBuff, nAPBuff = UnitAttackPower("player")
		local zAPwr = format("%.f", (bAPwr + pAPBuff + nAPBuff))
		if pAPBuff - nAPBuff < 0 then
			zAPwr = CR:Red(format("%.f", (bAPwr + pAPBuff + nAPBuff)))
		elseif pAPBuff - nAPBuff > 0 then
			zAPwr = CR:Green(format("%.f", (bAPwr + pAPBuff + nAPBuff)))
		end
		--Melee Attack Power Bonus
		local zAPB = " (+"..format("%.1f", ((bAPwr + pAPBuff + nAPBuff) / 14)).."dps)"
		--Melee Haste
		local bHasteRate = GetCombatRating(18)
		local zHaste = format("%.2f%%", GetCombatRatingBonus(18))
		if bHasteRate > 0 then zHaste = CR:Green(zHaste) end	
		--Expertise
		local bExpertiseRate = GetCombatRating(24)
		local mExpertise, oExpertise = GetExpertise()
		if bExpertiseRate > 0 then 
			mExpertise = CR:Green(mExpertise)
			oExpertise = CR:Green(oExpertise)		
		end
		local percent_mExpertise, percent_oExpertise = GetExpertisePercent()
		local zExpertise = mExpertise
		local zpExpertise = " ("..format("%.2f%%", percent_mExpertise)..")"
		if OffhandHasWeapon() then
			zExpertise = zExpertise.."/"..oExpertise
			zpExpertise = " ("..format("%.2f%%", percent_mExpertise).."/"..format("%.2f%%", percent_oExpertise)..")"  
		end

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Melee Skills"], mHeaderFont, "CENTER", 5)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Damage: "])
		tooltip:SetCell(line, 2, zDmgRng)
		tooltip:SetCell(line, 4, L["Crit Chance: "])
		tooltip:SetCell(line, 5, zCrit)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Spd: "])
		tooltip:SetCell(line, 2, zWeapSpeed)	
		tooltip:SetCell(line, 4, L["Hit Chance: "])
		tooltip:SetCell(line, 5, zHitRate)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["DPS: "])
		tooltip:SetCell(line, 2, zDPS)	
		tooltip:SetCell(line, 4, L["Atk Pwr: "])
		tooltip:SetCell(line, 5, zAPwr..zAPB)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Haste: "])
		tooltip:SetCell(line, 2, zHaste)	
		tooltip:SetCell(line, 4, L["Expertise: "])
		tooltip:SetCell(line, 5, zExpertise..zpExpertise)

		tooltip:AddSeparator()	
	end

-- RANGED
	if db.tooltip.show_Ranged then
		local zrSpd, zrDmgRng, zrDPS = "N/A", "N/A", "N/A"
		--Ranged Weapon Speed
		local rSpd, rLowDmg, rHiDmg, pDmgBuff, nDmgBuff, percentMod  = UnitRangedDamage("player")
		if not (rSpd == 0) then
			zrSpd = format("%.2f", rSpd)
			--Ranged Damage Range
			local trLowDmg = floor(rLowDmg)
			local trHiDmg = ceil(rHiDmg)
			zrDmgRng = format("%.f-%.f", trLowDmg, trHiDmg)
			if pDmgBuff - nDmgBuff < 0 then
				zrDmgRng = CR:Red(format("%.f-%.f", (trLowDmg + pDmgBuff + nDmgBuff), (trHiDmg + pDmgBuff + nDmgBuff)))
				elseif pDmgBuff - nDmgBuff > 0 then
				zrDmgRng = CR:Green(format("%.f-%.f", (trLowDmg + pDmgBuff + nDmgBuff), (trHiDmg + pDmgBuff + nDmgBuff)))
			end
			--Ranged Damage Per Second
			zrDPS = format("%.1f", (((trLowDmg + trHiDmg) / 2) / rSpd))
		end
		--Ranged Crit Chance Percentage
		local zrCrit = format("%.2f%%", GetRangedCritChance())
		--Ranged Hit Percent Chance versus Same Level NPC/PC
		local zrHitRate = "+"..format("%.2f%%", GetCombatRatingBonus(7))
		--Ranged Attack Power
		local bRngAPwr, pRngBuff, nRngBuff = UnitRangedAttackPower("player")
		local zRngAPwr = format("%.f", (bRngAPwr + pRngBuff + nRngBuff))
		if pRngBuff - nRngBuff < 0 then
			zRngAPwr = CR:Red(format("%.f", (bRngAPwr + pRngBuff + nRngBuff)))
		elseif pRngBuff - nRngBuff > 0 then
			zRngAPwr = CR:Green(format("%.f", (bRngAPwr + pRngBuff + nRngBuff)))
		end
		--Ranged Attack Power Bonus
		local zrAPB = ""
		if not ((bRngAPwr + pRngBuff + nRngBuff) == 0) then
			zrAPB = " (+"..format("%.1f", ((bRngAPwr + pRngBuff + nRngBuff) / 14)).."dps)"
		end
		--Range Haste
		local brHasteRate = GetCombatRating(19)
		local zrHaste = format("%.2f%%", GetCombatRatingBonus(19))
		if brHasteRate > 0 then zrHaste = CR:Green(zrHaste) end	


		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Ranged Skills"], mHeaderFont, "CENTER", 5)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Damage: "])
		tooltip:SetCell(line, 2, zrDmgRng)
		tooltip:SetCell(line, 4, L["Crit Chance: "])
		tooltip:SetCell(line, 5, zrCrit)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Spd: "])
		tooltip:SetCell(line, 2, zrSpd)	
		tooltip:SetCell(line, 4, L["Hit Chance: "])
		tooltip:SetCell(line, 5, zrHitRate)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["DPS: "])
		tooltip:SetCell(line, 2, zrDPS)	
		tooltip:SetCell(line, 4, L["Atk Pwr: "])
		tooltip:SetCell(line, 5, zRngAPwr..zrAPB)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Haste: "])
		tooltip:SetCell(line, 2, zrHaste)	
		tooltip:SetCell(line, 4, "")
		tooltip:SetCell(line, 5, "")

		tooltip:AddSeparator()	
	end

-- SPELL
	if db.tooltip.show_Spell then
		--Bonus Damage
		local zBDmg = GetSpellBonusDamage(2)
		--Bonus Healing
		local zBHeal = GetSpellBonusHealing()
		--Mana Regeneration
		local baseMR, castingMR = GetManaRegen()
		local zMRegen = format("%.f", floor(baseMR * 5))
		local zcRegen = format("%.f", floor(castingMR * 5))
		--Spell Crit Rate Percentage
		local zsCrit = GetRealSpellCrit()
		--Spell Hit Percent Chance versus Same Level NPC/PC
		local zsHitRate = "+"..format("%.2f%%", GetCombatRatingBonus(8))
		--Spell Penetration
		local zsPen = GetSpellPenetration()
		--Spell Haste
		local bsHasteRate = GetCombatRating(20)
		local zsHaste = format("%.2f%%", GetCombatRatingBonus(20))
		if bsHasteRate > 0 then zsHaste = CR:Green(zsHaste) end		

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Spell Skills"], mHeaderFont, "CENTER", 5)

		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Bonus Damage: "])
		tooltip:SetCell(line, 2, zBDmg)	
		tooltip:SetCell(line, 4, L["Crit Chance: "])
		tooltip:SetCell(line, 5, zsCrit)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Bonus Healing: "])
		tooltip:SetCell(line, 2, zBHeal)	
		tooltip:SetCell(line, 4, L["Hit Chance: "])
		tooltip:SetCell(line, 5, zsHitRate)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Haste: "])
		tooltip:SetCell(line, 2, zsHaste)	
		tooltip:SetCell(line, 4, L["Penetration: "])
		tooltip:SetCell(line, 5, zsPen)
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Mana Regen: "])
		tooltip:SetCell(line, 2, zMRegen)
		tooltip:SetCell(line, 4, L["Combat Regen: "])
		tooltip:SetCell(line, 5, zcRegen)	
		
		tooltip:AddSeparator()
	end

-- TALENTS, PROFESSIONS

	if db.tooltip.show_Talents then
		local n1N, n1Tal = GetPlayerTalentInfo(1)
		local n2N, n2Tal = GetPlayerTalentInfo(2)
		local n3N, n3Tal = GetPlayerTalentInfo(3)		
		local allPoints = GetNumTalentPoints()
		local freePoints = allPoints - (n1Tal + n2Tal + n3Tal)
		if freePoints > 0 then freePoints = CR:Green(freePoints) end
		local nextLevel = GetNextTalentLevel()

		local prof1_name, prof1_skill, prof1_maxskill = CR:Red(L["Profession #1"]), "", ""
		local prof2_name, prof2_skill, prof2_maxskill = CR:Red(L["Profession #2"]), "", ""
		local archaeology_name, archaeology_skill, archaeology_maxskill = CR:Red(GetSpellInfo(78670)), "", ""
		local cooking_name, cooking_skill, cooking_maxskill = CR:Red(GetSpellInfo(2550)), "", ""
		local firstAid_name, firstAid_skill, firstAid_maxskill = CR:Red(GetSpellInfo(3273)), "", ""
		local fishing_name, fishing_skill, fishing_maxskill = CR:Red(GetSpellInfo(7620)), "", ""
		
		local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions()
		
		if(prof1 ~= nil) then 
			prof1_name, prof1_skill, prof1_maxskill = GetPlayerProfessionInfo(prof1)
			prof1_name = CR:Gold(prof1_name)
		end
		if(prof2 ~= nil) then 
			prof2_name, prof2_skill, prof2_maxskill = GetPlayerProfessionInfo(prof2)
			prof2_name = CR:Gold(prof2_name)
		end	
		if(archaeology ~= nil) then archaeology_name, archaeology_skill, archaeology_maxskill = GetPlayerProfessionInfo(archaeology) end
		if(cooking ~= nil) then cooking_name, cooking_skill, cooking_maxskill = GetPlayerProfessionInfo(cooking) end
		if(firstAid ~= nil) then firstAid_name, firstAid_skill, firstAid_maxskill = GetPlayerProfessionInfo(firstAid) end		
		if(fishing ~= nil) then fishing_name, fishing_skill, fishing_maxskill = GetPlayerProfessionInfo(fishing) end
		
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Talents Info"], mHeaderFont, "CENTER", 2)
		tooltip:SetCell(line, 4, L["Professions Info"], mHeaderFont, "CENTER", 2)
		
		line = tooltip:AddLine()
		if pLVL > 9 then			
			tooltip:SetCell(line, 1, L["Talent Points: "])
			tooltip:SetCell(line, 2, allPoints)
		else	
			tooltip:SetCell(line, 1, CR:Red(L["You have not reached"]), "CENTER", 2)
		end	
		tooltip:SetCell(line, 4, prof1_name)
		tooltip:SetCell(line, 5, prof1_skill..prof1_maxskill)
	
		line = tooltip:AddLine()
		if pLVL > 9 then			
			tooltip:SetCell(line, 1, L["Free Talent Points: "])
			tooltip:SetCell(line, 2, freePoints)
		else			
			tooltip:SetCell(line, 1, CR:Red(L["level 10 yet."]), "CENTER", 2)
		end
		tooltip:SetCell(line, 4, prof2_name)
		tooltip:SetCell(line, 5, prof2_skill..prof2_maxskill)		
			
		line = tooltip:AddLine()
		if pLVL > 9 then			
			tooltip:SetCell(line, 1, n1N..": ")
			tooltip:SetCell(line, 2, n1Tal)	
		end	
		tooltip:SetCell(line, 4, archaeology_name)
		tooltip:SetCell(line, 5, archaeology_skill..archaeology_maxskill)
		
		line = tooltip:AddLine()
		if pLVL > 9 then			
			tooltip:SetCell(line, 1, n2N..": ")
			tooltip:SetCell(line, 2, n2Tal)	
		end	
		tooltip:SetCell(line, 4, cooking_name)
		tooltip:SetCell(line, 5, cooking_skill..cooking_maxskill)
		
		line = tooltip:AddLine()
		if pLVL > 9 then			
			tooltip:SetCell(line, 1, n3N..": ")
			tooltip:SetCell(line, 2, n3Tal)
		end		
		tooltip:SetCell(line, 4, firstAid_name)
		tooltip:SetCell(line, 5, firstAid_skill..firstAid_maxskill)
		
		line = tooltip:AddLine()
		if nextLevel then
			tooltip:SetCell(line, 1, L["Next Talent Point: "])
			tooltip:SetCell(line, 2, nextLevel.." lvl")	
		end
		tooltip:SetCell(line, 4, fishing_name)
		tooltip:SetCell(line, 5, fishing_skill..fishing_maxskill)		
		
		tooltip:AddSeparator()	

	end

	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "                    ")
	tooltip:SetCell(line, 2, "               ")
	tooltip:SetCell(line, 3, "      ")		
	tooltip:SetCell(line, 4, "                    ")
	tooltip:SetCell(line, 5, "               ")
		

	if not db.tooltip.hide_Hint then
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Left-click to open Character Info."], "LEFT", 5)
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Shift+Left-click to open Talents."], "LEFT", 5)
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, L["Right-click for options."], "LEFT", 5)
	end
	updater.elapsed = 0
	tooltip:Show()
end

function mPlayerInfo:Update()
	local _, _, _, _, color_text = GetPlayerData()
	DataObj.text = color_text
	if tooltip and tooltip:IsVisible() then
		DrawTooltip(LDB_anchor)
	end
end
--
-- Event functions
function mPlayerInfo:OnInitialize()
	-- Database
	local temp_db = LibStub("AceDB-3.0"):New(ADDON_NAME.."DB", defaults)
	db = temp_db.global
	self:SetupOptions()
end

function mPlayerInfo:OnEnable()
	self:RegisterEvent("CHAT_MSG_SKILL",self.Update)
	self:RegisterEvent("PLAYER_ENTERING_WORLD",self.Update)
	self:RegisterEvent("PLAYER_XP_UPDATE",self.Update)
	self:RegisterEvent("PLAYER_LEVEL_UP",self.Update)
	self:RegisterEvent("SPELLS_CHANGED",self.Update)
	self:RegisterEvent("UNIT_AURA",self.Update)
	self:RegisterEvent("UNIT_AURASTATE",self.Update)
	self:RegisterEvent("UNIT_INVENTORY_CHANGED",self.Update)
	
	DataObj = LDB:NewDataObject(ADDON_NAME, {
		type	= "data source",
		label	= ADDON_NAME,
		text	= " ",
		icon	= "Interface\\AddOns\\mPlayerInfo\\icons\\playerfu",		
		OnEnter	= LDB_OnEnter,
		OnLeave	= LDB_OnLeave,
		OnClick	= LDB_OnClick,
	})

	if LDBIcon then
		LDBIcon:Register(ADDON_NAME, DataObj, db.datafeed.minimap_icon)
	end
		
	startXP, startTime = UnitXP("player"), GetTime()
	startLevel = UnitLevel("player") + startXP/UnitXPMax("player")	
	
	self:Update()
end
--
-- Configuration.
local options
local function GetOptions()
	if not options then
		options = {
			name = ADDON_NAME,
			childGroups = "tab",
			type = "group",
			args = {
				datafeed = {
					name	= L["Datafeed"],
					order	= 2,
					type	= "group",
					args = {
						minimap_icon = {
							order	= 1,
							type	= "toggle",
							width	= "full",
							name	= L["Minimap Icon"],
							desc	= L["Draws the icon on the minimap."],
							get	= function()
									  return not db.datafeed.minimap_icon.hide
								  end,
							set	= function(info, value)
									  db.datafeed.minimap_icon.hide = not value
									  LDBIcon[value and "Show" or "Hide"](LDBIcon, ADDON_NAME)
								  end,
						},
						show_PlayerN = {
							order	= 2,
							type	= "toggle",
							width	= "full",
							name	= L["Show Name"],
							desc	= L["Toggle showing your name."],
							get	= function()
									  return db.datafeed.show_PlayerN
								  end,
							set	= function(info, value)
									  db.datafeed.show_PlayerN = value
									  mPlayerInfo:Update()
								  end,
						},
						show_PlayerL = {
							order	= 3,
							type	= "toggle",
							width	= "full",
							name	= L["Show Level"],
							desc	= L["Toggle showing your level."],
							get	= function()
									  return db.datafeed.show_PlayerL
								  end,
							set	= function(info, value)
									  db.datafeed.show_PlayerL = value
									  mPlayerInfo:Update()
								  end,
						},
						use_PlayerCC = {
							order	= 4,
							type	= "toggle",
							width	= "full",
							name	= L["Use Class Color"],
							desc	= L["Use class color for your name and level."],
							get	= function()
									  return db.datafeed.use_PlayerCC
								  end,
							set	= function(info, value)
									  db.datafeed.use_PlayerCC = value
									  mPlayerInfo:Update()
								  end,
						},						
						
						show_PlayerXPP = {
							order	= 5,
							type	= "toggle",
							width	= "full",
							name	= L["Show Percentage"],
							desc	= L["Toggle showing your current experience percentage until the next level (if below level 80)."],
							get	= function()
									  return db.datafeed.show_PlayerXPP
								  end,
							set	= function(info, value)
									  db.datafeed.show_PlayerXPP = value
									  mPlayerInfo:Update()
								  end,
						},
						show_PlayerXPN = {
							order	= 6,
							type	= "toggle",
							width	= "full",
							name	= L["Show Number"],
							desc	= L["Toggle showing the full text numbers of your current experience (if below level 80)."],
							get	= function()
									  return db.datafeed.show_PlayerXPN
								  end,
							set	= function(info, value)
									  db.datafeed.show_PlayerXPN = value
									  mPlayerInfo:Update()									  
								  end,
						},
						show_PlayerXPR = {
							order	= 7,
							type	= "toggle",
							width	= "full",
							name	= L["Show Rested"],
							desc	= L["Toggle showing your current rested experience (if below level 80)."],
							get	= function()
									  return db.datafeed.show_PlayerXPR
								  end,
							set	= function(info, value)
									  db.datafeed.show_PlayerXPR = value
									  mPlayerInfo:Update()									  
								  end,
						},
						show_PlayerXPRe = {
							order	= 8,
							type	= "toggle",
							width	= "full",
							name	= L["Show Remaining"],
							desc	= L["Toggle showing the full text numbers of your current experience until level (if below level 80)."],
							get	= function()
									  return db.datafeed.show_PlayerXPRe
								  end,
							set	= function(info, value)
									  db.datafeed.show_PlayerXPRe = value
									  mPlayerInfo:Update()									  
								  end,
						},						
						
					},
				},
				tooltip = {
					name = L["Tooltip"],
					order = 3,
					type = "group",
					args = {
						hide_Hint = {
							order	= 1,
							type	= "toggle",
							width	= "full",
							name	= L["Hide Hint Text"],
							desc	= L["Hides the hint text at the bottom of the tooltip."],
							get	= function()
									  return db.tooltip.hide_Hint
								  end,
							set	= function(info, value)
									  db.tooltip.hide_Hint = value
								  end,
						},
						show_Exp = {
							order	= 2,
							type	= "toggle",
							width	= "full",
							name	= L["Show Experience"],
							desc	= L["Toggle showing your experience info."],
							get	= function()
									  return db.tooltip.show_Exp
								  end,
							set	= function(info, value)
									  db.tooltip.show_Exp = value
								  end,
						},
						show_Attributes = {
							order	= 3,
							type	= "toggle",
							width	= "full",
							name	= L["Show Attributes"],
							desc	= L["Toggle showing your attributes info."],
							get	= function()
									  return db.tooltip.show_Attributes
								  end,
							set	= function(info, value)
									  db.tooltip.show_Attributes = value
								  end,
						},
						show_Defense = {
							order	= 4,
							type	= "toggle",
							width	= "full",
							name	= L["Show Defense"],
							desc	= L["Toggle showing your defense info."],
							get	= function()
									  return db.tooltip.show_Defense
								  end,
							set	= function(info, value)
									  db.tooltip.show_Defense = value
								  end,
						},
						show_Melee = {
							order	= 5,
							type	= "toggle",
							width	= "full",
							name	= L["Show Melee"],
							desc	= L["Toggle showing your melee info."],
							get	= function()
									  return db.tooltip.show_Melee
								  end,
							set	= function(info, value)
									  db.tooltip.show_Melee = value
								  end,
						},
							show_Ranged = {
							order	= 6,
							type	= "toggle",
							width	= "full",
							name	= L["Show Ranged"],
							desc	= L["Toggle showing your ranged info."],
							get	= function()
									  return db.tooltip.show_Ranged
								  end,
							set	= function(info, value)
									  db.tooltip.show_Ranged = value
								  end,
						},					
							show_Spell = {
							order	= 7,
							type	= "toggle",
							width	= "full",
							name	= L["Show Spell"],
							desc	= L["Toggle showing your spell info."],
							get	= function()
									  return db.tooltip.show_Spell
								  end,
							set	= function(info, value)
									  db.tooltip.show_Spell = value
								  end,
						},						
							show_Talents = {
							order	= 8,
							type	= "toggle",
							width	= "full",
							name	= L["Show Talents and Profession"],
							desc	= L["Toggle showing your talents and profession info."],
							get	= function()
									  return db.tooltip.show_Talents
								  end,
							set	= function(info, value)
									  db.tooltip.show_Talents = value
								  end,
						},
						scale = {
							order	= 9,
							type	= "range",
							width	= "full",
							name	= L["Tooltip Scale"],
							desc	= L["Move the slider to adjust the scale of the tooltip."],
							min	= 0.5,
							max	= 1.5,
							step	= 0.01,
							get	= function()
									  return db.tooltip.scale
								  end,
							set	= function(info, value)
									  db.tooltip.scale = math.max(0.5, math.min(1.5, value))
								  end,
						},
						timer = {
							order	= 10,
							type	= "range",
							width	= "full",
							name	= L["Tooltip Timer"],
							desc	= L["Move the slider to adjust the tooltip fade time."],
							min	= 0.1,
							max	= 2,
							step	= 0.01,
							get	= function()
									  return db.tooltip.timer
								  end,
							set	= function(info, value)
									  db.tooltip.timer = math.max(0.1, math.min(2, value))
								  end,
						},
					}
				}
			}
		}
	end
	return options
end

function mPlayerInfo:SetupOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable(ADDON_NAME, GetOptions())
	self.options_frame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(ADDON_NAME)
end
